//-----------------------------------------------------------------------------
// Title      : AsyncArt micropipeline demo for iCEstick
// Project    : AsyncArt
// URL        : https://www.ohwr.org/project/asyncart
//-----------------------------------------------------------------------------
// File       : micropipeline.v
// Author(s)  : Javier Garcia Lasheras <jgarcia@gl-research.com>
// Company    : GL Research Corporation SLU
//-----------------------------------------------------------------------------
// Description: Demonstrative micropipeline demo for the iCEstick FPGA board.
//-----------------------------------------------------------------------------
// Copyright (c) 2019 GL Research Corporation SLU
//-----------------------------------------------------------------------------
// APACHE LICENSE, VERSION 2.0
//-----------------------------------------------------------------------------

/*
 * This micropipeline demo for the iCEstick is based on:
 *
 *    https://github.com/cyrozap/iCEstick-UART-Demo
 *
 * Copyright 2015 Forest Crossman <cyrozap@gmail.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

`include "../../ip_cores/osdvu/uart.v"
`include "../../hdl/verilog/asyncart_demo.v"

module top(
	input iCE_CLK,
	input RS232_Rx_TTL,
	output RS232_Tx_TTL,
	output LED0,
	output LED1,
	output LED2,
	output LED3,
	output LED4
	);


	wire reset = 0;
	wire enable = 1;

	wire received;
	reg fetched;
	wire [7:0] rx_byte;
	wire is_receiving;
	wire is_transmitting;
	wire recv_error;

        wire [31:0] data_micropipeline;
        reg act, rst;


        uart #(
                .baud_rate(115200),               // The baud rate in kilobits/s
                .sys_clk_freq(12000000)           // The master clock frequency
        )
        uart0(
                .clk(iCE_CLK),                    // The master clock for this module
                .rst(reset),                      // Synchronous reset
                .rx(RS232_Rx_TTL),                // Incoming serial line
                .tx(RS232_Tx_TTL),                // Outgoing serial line
                .transmit(transmit),              // Signal to transmit
                .tx_byte(tx_byte),                // Byte to transmit
                .received(received),              // Indicated that a byte has been received
                .rx_byte(rx_byte),                // Byte received
                .is_receiving(is_receiving),      // Low when receive line is idle
                .is_transmitting(is_transmitting),// Low when transmit line is idle
                .recv_error(recv_error)           // Indicates error in receiving packet.
        );

        always @(posedge iCE_CLK) begin
                if (received) begin
                        act  <= rx_byte[1];
                        rst  <= rx_byte[0];
                end
        end


        asyncart_demo #(
            .micropipeline_depth(4)
        )
        asyncart_demo_instance (
            .act_in(act),
            .rst_in(rst),
            .data_out(data_micropipeline)
        );



        // Connect the Red LEDs to the 4 MSB bits in the sink registed (last pipeline stage)
	assign {LED3, LED2, LED1, LED0} = data_micropipeline[31:28];
	assign LED4 = act;

endmodule
