//-----------------------------------------------------------------------------
// Title      : AsyncArt register rendezvous cell
// Project    : AsyncArt
// URL        : https://www.ohwr.org/project/asyncart
//-----------------------------------------------------------------------------
// File       : asyncart_reg.v
// Author(s)  : Javier Garcia Lasheras <jgarcia@gl-research.com>
// Company    : GL Research Corporation SLU
//-----------------------------------------------------------------------------
// Description: Rendezvous cell for implementing micropipeline register stages.
//-----------------------------------------------------------------------------
// Copyright (c) 2006-2019 GL Research Corporation SLU
//-----------------------------------------------------------------------------
// GNU LESSER GENERAL PUBLIC LICENSE
//-----------------------------------------------------------------------------
// This source file is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2.1 of the License, or (at your
// option) any later version. This source is distributed in the hope that it
// will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU Lesser General Public License for more details. You should have
// received a copy of the GNU Lesser General Public License along with this
// source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
//-----------------------------------------------------------------------------

module asyncart_reg (
    output fire_out, 
    output phase_out,
    input phase_in_pos,
    input phase_in_neg,
    input act,
    input rst);

reg phase;
wire fire;

always @(posedge fire or posedge rst)
begin
    if (rst) 
            phase <= 1'b0 ;
    else
            phase <= ! phase;
end

assign fire = (phase_in_pos ^ phase) & (phase_in_neg ~^ phase) & act;
assign phase_out = phase;
assign fire_out = fire;

endmodule
