-------------------------------------------------------------------------------
-- Title      : AsyncArt register rendezvous cell
-- Project    : AsyncArt
-- URL        : https://www.ohwr.org/project/asyncart
-------------------------------------------------------------------------------
-- File       : asyncart_reg.vhd
-- Author(s)  : Javier Garcia Lasheras <jgarcia@gl-research.com>
-- Company    : GL Research Corporation SLU
-------------------------------------------------------------------------------
-- Description: Rendezvous cell for implementing micropipeline register stages.
-------------------------------------------------------------------------------
-- Copyright (c) 2006-2019 GL Research Corporation SLU
-------------------------------------------------------------------------------
-- GNU LESSER GENERAL PUBLIC LICENSE
-------------------------------------------------------------------------------
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;

entity asyncart_reg is
    port (
        fire_out : out std_logic;
        phase_out : out std_logic;
        phase_in_pos : in std_logic;
        phase_in_neg : in std_logic;
        act : in std_logic;
        rst : in std_logic);
end entity;

architecture design of asyncart_reg is

signal fire, phase, phase_next : std_logic;

begin

    reg_fifo: process (rst, fire)
    begin
        if (rst = '1') then
            phase <=  '0';
        else
            if (fire'event and fire = '1') then
                phase <=  phase_next;
            end if;
        end if;
    end process reg_fifo;

    fire <= (phase_in_pos xor phase) and (phase_in_neg xnor phase) and act;
    phase_next <= not phase;
    phase_out <= phase;
    fire_out <= fire;

end design;

